<?php

namespace Drupal\config_enforce_devel\Form;

use Drupal\config_enforce\ConfigEnforce;
use Drupal\config_enforce\Form\AbstractConfigEnforceForm;
use Drupal\config_enforce_devel\ConfigEnforceDevel;
use Drupal\config_enforce_devel\Form\ConfigEnforceDevelFormTrait;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Defines a settings form for the Config Enforce Devel module.
 */
class ConfigEnforceDevelEmbeddedForm extends AbstractConfigEnforceForm {

  use ConfigEnforceDevelFormTrait;

  const FORM_ID = 'config_enforce_devel_embedded';

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $this->setSharedObjectProperties($form, $form_state);

    $this->makeFormFloat();

    if ($this->hasKnownIssues()) {
      $this->addKnownIssuesWarning();
    }

    $this->addConfigFormInfo();

    return parent::buildForm($this->form, $this->formState);
  }

  /**
   * Return a warning that the host form may be buggy.
   */
  protected function addKnownIssuesWarning() {
    $issues = [];
    foreach ($this->getKnownIssues() as $id => $title) {
      $url = Url::fromUri('https://www.drupal.org/node/' . $id);
      $link = Link::fromTextAndUrl('Issue #' . $id . ': ' . $title, $url);
      $issues[] = $link;
    }

    $issue_list = [
      '#theme' => 'item_list',
      '#items' => $issues,
    ];

    $this->addWarning(
      'This configuration form (%form_id) may not yet be fully supported.<br /> For more information, review the following @issue: @issue_list',
      [
        '%form_id' => $this->getContext('form_id'),
        '@issue' => count($issues) > 1 ? $this->t('issues') : $this->t('issue'),
        '@issue_list' => drupal_render($issue_list),
      ]
    );
  }

  /**
   * Determine whether the host form has any known issues.
   */
  protected function hasKnownIssues() {
    return array_key_exists($this->getContext('form_id'), $this->getAllKnownIssues());
  }

  /**
   * Return a list of all known issues for the host form.
   */
  protected function getKnownIssues() {
    return $this->getAllKnownIssues()[$this->getContext('form_id')];
  }

  /**
   * Return a list of all known issues for all forms that are know to be buggy.
   */
  protected function getAllKnownIssues() {
    return [
      'view_edit_form' => [
        '3164882' => 'Fix broken config entity forms',
      ],
      'block_form' => [
        '3164882' => 'Fix broken config entity forms',
      ],
    ];
  }

}
