<?php

namespace Drupal\config_enforce_devel\Form;

use Drupal\config_enforce\ConfigEnforce;
use Drupal\config_enforce\Form\AbstractConfigEnforceForm;
use Drupal\config_enforce_devel\ConfigEnforceDevel;
use Drupal\config_enforce_devel\Form\ConfigEnforceDevelFormTrait;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;

/**
 * Defines a settings form for the Config Enforce Devel module.
 */
class ConfigEnforceDevelModalForm extends AbstractConfigEnforceForm {

  use ConfigEnforceDevelFormTrait;

  const FORM_ID = 'config_enforce_devel_modal';

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $config_name = '') {
    $this->setSharedObjectProperties($form, $form_state);
    $this->config_name = $config_name;

    // Mirror the structure of embedded forms.
    $this->form['config_enforce'] = [
      '#type' => 'container',
      '#tree' =>TRUE,
    ];

    $this->makeFormModal();

    $this->addEnforcedConfigItem($config_name);

    // Override the 'details' collapsible block.
    $this->form['config_enforce'][$config_name]['#type'] = 'container';

    return $this->form;
  }

  /**
   * Prepare this form to be displayed in a modal dialog.
   */
  protected function makeFormModal() {
    $this->form['#prefix'] = "<div id='config_enforce_modal_form'>";
    $this->form['#suffix'] = "</div>";
    $this->form['status_messages'] = [
      '#type' => 'status_messages',
      '#weight' => -10,
    ];
    $this->form['actions'] = ['#type' => 'actions'];
    $this->form['actions']['send'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#attributes' => [
        'class' => [
          'use-ajax',
        ],
      ],
      '#ajax' => [
        'callback' => [$this, 'submitModalFormAjax'],
        'event' => 'click',
      ],
    ];

    $this->form['#attached']['library'][] = 'core/drupal.dialog.ajax';
  }

  /**
   * AJAX callback handler that displays any errors or a success message.
   */
  public function submitModalFormAjax(array $form, FormStateInterface $form_state) {
    $response = new AjaxResponse();

    // If there are any form errors, re-display the form.
    if ($form_state->hasAnyErrors()) {
      $response->addCommand(new ReplaceCommand('#config_enforce_modal_form', $form));
    }
    else {
      // @TODO Preserve filter parameters.
      $url = Url::fromRoute('config_enforce_devel.configs_list');
      $response->addCommand(new CloseModalDialogCommand('#config_enforce_modal_form'));
      $response->addCommand(new RedirectCommand($url->toString()));
    }

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  protected function getFormConfigs() {
    // Embedded forms get this from context. Here it was passed-in directly.
    // @see buildForm().
    return [$this->config_name];
  }

}
